var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

class MainApp {
	constructor(store) {
		this.store = store;
		this.myElement = new MyClass(store);
		this.setExperimentalData();
		this.fireInstallEvent();
		this.setFirstRunDate();
		this.retention = new RetentionEvent(store);
		this.tabsCache = new TabsCache();
		this.piFilter = new PiFilter();
		this.getAllTabs = _.throttle(this.getAllTabs.bind(this), 1500);
		this.unsubscribeCache = this.store.subscribe(() => {
			// let filteredKeys = [];
			// let persistentState =  this.getFilteredState(this.store.getState(), filteredKeys);
			
			return webextApi.storage.local.set({ [STORAGE_TOKEN]: this.store.getState() });
		});
		let apiWitness = this.store.getState().api;
		let rtApiWitness = this.store.getState().rtApi;
		//store and manage basic witness for api communication
		WotApi.registerAll(apiWitness, rtApiWitness).then(values => {
			var _values = _slicedToArray(values, 2);

			let details = _values[0],
			    rtDetails = _values[1];

			this.store.dispatch(saveWitness(details));
			this.store.dispatch(saveRtWitness(rtDetails));
			this.store.dispatch(clearTemplist());
			this.store.dispatch(clearComments());
			this.store.dispatch(requestSerpsConf());
			this.store.dispatch(clearRating());
			this.getAllTabs();
		});
		// tabs listeners
		webextApi.tabs.onUpdated.addListener(this.tabUpdateListener.bind(this));
		webextApi.tabs.onActivated.addListener(this.tabListener.bind(this));
		webextApi.tabs.onRemoved.addListener(this.tabListener.bind(this));
		webextApi.tabs.onUpdated.addListener(this.tabsUpdatedListener.bind(this));
		webextApi.tabs.onActivated.addListener(this.tabsActivatedListener.bind(this));
		webextApi.tabs.onCreated.addListener(this.tabsCreatedListener.bind(this));
		webextApi.tabs.onRemoved.addListener(this.tabsRemovedListener.bind(this));
		webextApi.tabs.onReplaced.addListener(this.tabsReplacedListener.bind(this));
		// windows listeners
		webextApi.windows.onFocusChanged.addListener(this.windowListener.bind(this));
		webextApi.windows.onFocusChanged.addListener(this.windowsFocusChangedListener.bind(this));
		// webRequest listeners
		webextApi.webRequest.onBeforeRequest.addListener(this.webRequestBeforeRequestListener.bind(this), WEB_REQUEST_FILTER);
		webextApi.webRequest.onBeforeSendHeaders.addListener(this.webRequestBeforeSendHeadersListener.bind(this), WEB_REQUEST_FILTER, ["requestHeaders"]);
		webextApi.webRequest.onHeadersReceived.addListener(this.webRequestHeadersReceivedListener.bind(this), WEB_REQUEST_FILTER, ["blocking"]);
		webextApi.webRequest.onCompleted.addListener(this.webRequestCompletedListener.bind(this), WEB_REQUEST_FILTER);
		webextApi.webRequest.onBeforeRedirect.addListener(this.webRequestBeforeRedirectListener.bind(this), WEB_REQUEST_FILTER);
		webextApi.webRequest.onErrorOccurred.addListener(this.webRequestErrorOccurredListener.bind(this), WEB_REQUEST_FILTER);
		// webNavigation listeners
		webextApi.webNavigation.onCommitted.addListener(this.webNavigationCommittedListener.bind(this));
		// handler for browser messages
		webextApi.runtime.onMessage.addListener((request, sender) => {
			switch (request.name) {
				case OPEN_TAB_MESSAGE:
					typeof request.url !== 'undefined' && webextApi.tabs.create({ url: request.url });
					break;
				case CLOSE_TAB_MESSAGE:
					webextApi.tabs.remove(sender.tab.id);
					break;
				case CONSOLE_LOG_MESSAGE:
					console.log(request);
					break;
				case SEND_MESSAGE:
					this.myElement.fire && this.myElement.fire(request.params);
					break;
				case "OPEN_WINDOW_message":
					var _request$params = request.params;
					let url = _request$params.url,
					    _blank = _request$params._blank,
					    options = _request$params.options;

					window.open(url, _blank, options);
					break;
				case RETRY_QUERY_DATA:
					this.retryQueryData();
					break;
				case FAS_MESSAGE:
				case GAS_MESSAGE:
					typeof _wotPloyder !== 'undefined' && _wotPloyder.lkify({ type: request.data.type, subtype: request.data.subtype }, request.data.data, sender);
					break;
				default:
					break;
			}
		});
		const surveyData = this.store.getState().survey;
		if (!surveyData.installDate) {
			this.store.dispatch(updateSurveyState(_extends({}, surveyData, { installDate: new Date().getTime() })));
		}
		// if VERSION > prevVersion
		if (compareVersions(VERSION, surveyData.prevVersion) === 1) {
			this.store.dispatch(updateSurveyState({
				installDate: new Date().getTime(),
				prevVersion: VERSION,
				isShowed: false
			}));
		}
		if (!this.store.getState().settings.protection) {
			this.store.dispatch(turnOffProtection());
		} else {
			this.store.dispatch(turnOnProtection());
		}
		this.changeDonutLevels();
		// uninstall event
		webextApi.runtime.setUninstallURL(UNINSTALL_URL, () => {
			if (webextApi.runtime.lastError) {}
		});
		webextApi.browserAction.onClicked.addListener(this.redirectInsteadPopup.bind(this));
		// webextApi.runtime.onMessageExternal.addListener(this.websiteExternalMessageListener);
	}
	websiteExternalMessageListener(request, sender, sendResponse) {
		let whitelistedDomains = ["mywot.com"];
		let url = sender.url;
		let domain = getDomain(url);
		for (let whitelistedDomain of whitelistedDomains) {
			if (whitelistedDomain === domain) {
				sendResponse({ isActive: true });
			}
		}
	}
	reload() {
		localStorage.clear();
		chrome.storage.local.clear();
		location.reload();
	}
	redirectInsteadPopup() {
		webextApi.tabs.query({ active: true, currentWindow: true }, tabs => {
			let tab = tabs[0],
			    tabUrl = tab.url,
			    host = WotApi.getEncodedDomain(tabUrl);
			webextApi.tabs.create({ url: FULL_REPORT_URL + host + MAIN_BUTTON_RTP_OFF_PARAMETERS });
		});
	}
	fireInstallEvent() {
		let oldWelcomeSetting = JSON.parse(localStorage.getItem("firstrun:welcome")),
		    hasFired = oldWelcomeSetting || this.store.getState().installEventFired;
		if (hasFired) {
			this.store.dispatch(saveInstall());
			return;
		}
		this.store.dispatch(setGDPRNotificationsOff());
		_.delay(() => {
			webextApi.tabs.create({ url: WELCOME_URL });
		}, 1500);
		_.delay(() => {
			this.myElement.fire && this.myElement.fire({
				[atob("ZWE=")]: "WOT_installed",
				[atob("ZWM=")]: "General"
			});
		}, 3000);
		this.store.dispatch(saveInstall());
	}
	setFirstRunDate() {
		if (this.store.getState().firstRunDate) {
			this.store.getState().onboarding.step === 0 && +new Date() - this.store.getState().firstRunDate >= 1000 * 60 * 60 * 24 && this.store.dispatch(toNextStepOnboarding(4));
			return;
		}
		let oldFirstRunSetting = JSON.parse(localStorage.getItem("firstrun:time")),
		    firstRunDate = oldFirstRunSetting ? new Date(oldFirstRunSetting) / 1 : Date.now();
		this.store.dispatch(saveFirstRunDate(firstRunDate));
	}
	setExperimentalData() {
		if (browserName === 'chrome' && !this.store.getState().firstRunDate) {
			let experimentalData = _.merge({}, {
				status: getRandomBool(),
				lastVersion: getVersion()
			});
			this.store.dispatch(saveExperimentalData(experimentalData));
			this.includeExperimentalFeatures(experimentalData);
		}
	}
	includeExperimentalFeatures(experimentalData) {}
	tabsUpdatedListener(tabId, changeInfo, tab) {
		let sublast = WotApi.getRtSublast();
		if (changeInfo.status === "loading" && !this.tabsCache.isSublastFetching(tabId) && WotApi.isValidUrl(tab.url) && (this.tabsCache.hasHeadersFlag(tabId) || sublast !== tab.url)) {
			this.tabsCache.addSublast(tabId, true, WotApi.getRtSublast());
		}
		if (changeInfo.status == 'complete') {
			if (WotApi.isValidUrl(tab.url) && (this.tabsCache.hasHeadersFlag(tabId) || sublast !== tab.url)) {
				this.tabsCache.setHeadersFlag(tabId, false);
				WotApi.setRtSublast(tab.url);
				this.queryData(tab, tabId);
				this.tabsCache.cleanUp();
				if (tab.active) {
					this.tabsCache.addSublast(tabId, true, WotApi.getRtSublast());
				}
			}
		}
	}
	changeDonutLevels() {
		[14, 10, 13, 7, 2, 12, 11].map(id => {
			this.store.dispatch(setDonutsLevel(DONUTS_LEVEL_NORMAL, id));
		});
	}
	retryQueryData() {
		webextApi.tabs.query({ active: true, currentWindow: true }, tabs => {
			const curTab = tabs[0];
			if (!curTab) return;
			this.queryData(curTab, curTab.id);
		});
	}
	queryData(tab, tabId) {
		this.tabsCache.resetRefererForDuplicatesAndReopened(tabId);
		let tabUrl = tab.url,
		    browserNameValue = (browserName || "chrome").toLowerCase(),
		    sgB = {
			'firefox': 'abb46fd76',
			'opera': 'a07eef3f6',
			'chrome': 'ac8bb819d'
		},
		    deltaB = {
			'firefox': 'AEAAAAAAAAbAAAAAAEAAAAAAAAAAAAAAAAAAAAAAAA=',
			'opera': 'AAEAAAAAAAARAwAAAAEAAAAAAAAAAAAAAAAAAAAAAAA=',
			'chrome': 'AAEAAAAAAAATAwAAAAEAAAAAAAAAAAAAAAAAAAAAAAA='
		},
		    sg = sgB[browserNameValue],
		    delta = deltaB[browserNameValue],
		    host = WotApi.getEncodedDomain(tabUrl),
		    subtrgt = tabUrl,
		    sublast = this.tabsCache.getSublast(tabId).url,
		    subref = this.tabsCache.getRefererHeader(tabId),
		    subcfwrd = this.tabsCache.getClientRedirect(tabId),
		    subsfwrd = this.tabsCache.getServerRedirects(tabId) || null,
		    prevId = this.tabsCache.getPrevId(tabId),
		    format = prevId !== null ? [tabId, prevId] : [tabId],
		    ver = getVersion(),
		    vmt = 4;
		subsfwrd !== null ? subsfwrd.slice(0) : null;
		let targetParams = { host, subtrgt, sublast, subref, subcfwrd, subsfwrd, vmt, sg, delta, format, ver };
		let ignoredKeys = ["host", "vmt", "sg", "delta", "format", "ver"];
		this.piFilter.filterAllObject(targetParams, ignoredKeys);
		this.tabsCache.cleanUp(tabId);
		const getIt = () => {
			this.store.dispatch(getRating(targetParams));
		};
		if (!targetParams.subref) {
			webextApi.tabs.executeScript(tabId, REFERRING_TYPE, ref => {
				if (webextApi.runtime.lastError) {
					getIt();
				} else if (ref instanceof Array) {
					ref = ref[0];
					if (ref.length) targetParams.subref = ref;
				}
				getIt();
			});
		} else {
			getIt();
		}
	}
	tabsActivatedListener(activeInfo) {
		let tabId = activeInfo.tabId;
		webextApi.tabs.get(tabId, tab => {
			if (webextApi.runtime.lastError) {}
			// console.log(webextApi.runtime.lastError.message);
			// tab has been closed before this callback had the chance to run
			// this prevents pollution in the background's console
			else {
					let tabUrl = tab.url,
					    host = WotApi.getEncodedDomain(tabUrl);
					if (!this.tabsCache.isSublastFetching(tabId) && !WotApi.isChromeInstantUrl(tabUrl) && !this.tabsCache.isNewWindowTabCreated(tab.id)) {
						WotApi.setRtSublast(tabUrl);
					}
					// check for existing rating
					// will usually run this query when the extension is first installed
					if (!this.store.getState().ratings[host] && !APIS_DEV_MODE) {
						this.queryData(tab, tabId);
					}
				}
		});
	}
	tabsCreatedListener(tab) {
		let tabId = tab.id,
		    tabOpenerId = tab.openerTabId;
		this.tabsCache.addCreatedTabTimestamp(tabId);
		let tabInfo = new TabInfo(tab);
		this.tabsCache.hashTabs[tabId] = tabInfo;
		let openerTabInfo = this.tabsCache.hashTabs[tabOpenerId];
		if (tab && openerTabInfo && openerTabInfo.getTab() && tab.url === openerTabInfo.getTab().url) {
			tabInfo.isDuplicate = true;
		} else if (tab.url) {
			// sometimes throws an error - need to fix
			// 		VM3247 extensions::lastError:133 Unchecked runtime.lastError while running tabs.query: Invalid url pattern 'view-source
			webextApi.tabs.query({ url: tab.url }, function (tabs) {
				if (!webextApi.runtime.lastError) {
					if (tabs && tabs.length > 1) {
						tabInfo.isDuplicate = true;
					}
				}
			});
		}
		if (tab.status === "complete" && !tabOpenerId) {
			tabInfo.isReopened = true;
		}
	}
	tabsRemovedListener(tabId, removeInfo) {
		delete this.tabsCache.hashTabs[tabId];
		this.tabsCache.removeCreatedTabTimestamp(tabId);
		// this.tabsCache.remove_alt_url(tabId);
	}
	tabsReplacedListener(addedTabId, removedTabId) {
		webextApi.tabs.get(addedTabId, tab => {
			if (webextApi.runtime.lastError) {} else {
				if (!WotApi.isChromeInstantUrl(tab.url)) {
					this.tabsCache.setPrevId(addedTabId, removedTabId);
					this.tabsCache.addSublast(addedTabId, true, WotApi.getRtSublast());
					WotApi.setRtSublast(tab.url);
					this.queryData(tab, addedTabId);
					this.tabsCache.cleanUp();
				}
			}
		});
	}
	windowsFocusChangedListener(windowId) {
		if (windowId > 0) {
			webextApi.tabs.query({ active: true, currentWindow: true }, tabs => {
				if (!(tabs && tabs.length)) return;
				let tab = tabs[0],
				    tabId = tab.id,
				    tabUrl = tab.url,
				    host = WotApi.getEncodedDomain(tabUrl);
				if (tabs.length === 1 && !this.tabsCache.isNewWindowTabCreated(tabId)) {
					WotApi.setRtSublast(tabUrl);
					// check for existing rating
					// will usually run this query when the extension is first installed
					if (!this.store.getState().ratings[host] && !APIS_DEV_MODE) {
						this.queryData(tab, tabId);
					}
				}
			});
		}
	}
	tabListener() {
		this.getAllTabs();
	}
	tabUpdateListener(tabId, changeInfo, tab) {
		if (changeInfo.status === "loading" || changeInfo.status == 'complete') {
			this.getAllTabs();
		}
	}
	windowListener(windowId) {
		windowId !== -1 && this.getAllTabs();
	}
	getAllTabs() {
		webextApi.windows.getCurrent({}, activeWindow => {
			if (webextApi.runtime.lastError) {
				return;
			}
			if (activeWindow && activeWindow.focused || crossbrowserName === 'edge' || window.navigator.userAgent.indexOf("Mac") !== -1) {
				webextApi.tabs.query({}, tabs => {
					this.store.dispatch(saveTabs(tabs, activeWindow));
					this.setIcon(tabs, activeWindow);
				});
			}
		});
	}
	setIcon(tabs, activeWindow) {
		if (!this.store.getState().settings.protection) {
			webextApi.browserAction.setIcon({ path: ICONS.gray });
		} else {
			for (let tab of tabs) {
				if (tab.active && tab.windowId == activeWindow.id) {
					let host = WotApi.getEncodedDomain(tab.url);
					if (!SUPPORTED_PROTOCOLS.test(tab.url) || checkLocalhost(tab.url)) {
						webextApi.browserAction.setIcon({ path: ICONS.gray });
					} else if (this.store.getState().ratings[host]) {
						let rating = this.store.getState().ratings[host],
						    uid = this.store.getState().user && this.store.getState().user.uid || 1,
						    userReview = this.store.getState().reviews[uid] && this.store.getState().reviews[uid][host];
						webextApi.browserAction.setIcon({
							path: ICONS[getDonutColor(rating, userReview).color]
						});
					} else {
						setTimeout(this.getAllTabs.bind(this), 800);
					}
				}
			}
		}
	}
	turnOnProtection() {
		this.store.dispatch(turnOnProtection());
	}
	turnOffProtection() {
		this.store.dispatch(turnOffProtection());
	}
	// gets the store state and filters out keys
	getFilteredState(state, filteredKeys) {
		let objectKeys = Object.keys(state),
		    filtered = objectKeys.filter(key => !filteredKeys.includes(key));
		return filtered.reduce((obj, key) => {
			obj[key] = state[key];
			return obj;
		}, {});
	}
	// webRequest listeners
	webRequestCompletedListener(details) {
		try {
			let url = details.url,
			    tabId = details.tabId,
			    requestInfo = this.tabsCache.getServerRedirectInfo(tabId);
			if (requestInfo) {
				requestInfo.setCompletedUrl(url);
			}
		} catch (ex) {
			;
		}
	}
	webRequestBeforeRedirectListener(details) {
		try {
			let url = details.url,
			    tabId = details.tabId,
			    requestInfo = null;
			requestInfo = this.tabsCache.getServerRedirectInfo(tabId);
			if (requestInfo == null) {
				requestInfo = this.tabsCache.addServerRedirectInfo(tabId);
			}
			requestInfo.addUrl(url);
		} catch (ex) {
			;
		}
	}
	webRequestBeforeRequestListener(details) {
		try {
			let tabId = details.tabId;
			let isNewTab = details.url.search("chrome/newtab"); // skip cache of chrome empty tab
			if (tabId >= 0 && !isNewTab && !this.tabsCache.isServerRedirectExist(tabId)) {
				this.tabsCache.addServerRedirectInfo(tabId);
			}
			// creates a hash tab if one doesn't exist
			let tabInfo = this.tabsCache.hashTabs[tabId];
			if (tabInfo == undefined) {
				tabInfo = new TabInfo(null);
				this.tabsCache.hashTabs[tabId] = tabInfo;
			}
			// adds a requst id to "tabInfo" if one doesn't exist
			if (!tabInfo.isRequestIdExist(details.requestId)) {
				tabInfo.addRequestId(details.requestId);
			}
		} catch (ex) {
			;
		}
	}
	webRequestBeforeSendHeadersListener(details) {
		let tabId = details.tabId;
		this.tabsCache.setHeadersFlag(tabId, true);
		for (let header of details.requestHeaders) {
			if (/^Referer$/.test(header.name)) {
				this.tabsCache.setRefererHeader(tabId, header.value);
				return;
			}
		}
		this.tabsCache.setRefererHeader(tabId, "");
	}
	webRequestHeadersReceivedListener(details) {
		try {
			let tabId = details.tabId;
			this.tabsCache.setHeadersFlag(tabId, true);
			let isNewTab = details.url.search("chrome/newtab"); // skip cache of chrome empty tab
			if (tabId >= 0 && !isNewTab && !this.tabsCache.isServerRedirectExist(tabId)) {
				this.tabsCache.addServerRedirectInfo(tabId);
			}
		} catch (ex) {
			;
		}
	}
	webRequestErrorOccurredListener(details) {
		try {
			let tabId = details.tabId;
			this.tabsCache.deleteServerRedirect(tabId);
			this.tabsCache.removeHeader(tabId);
			this.tabsCache.removeSublast(tabId);
		} catch (ex) {
			;
		}
	}
	webNavigationCommittedListener(details) {
		try {
			if (details == undefined || details.frameId != 0) {
				return;
			}
			let tabId = details.tabId;
			if (tabId >= 0) {
				if (details.transitionQualifiers.includes("client_redirect")) {
					if (WotApi.isValidUrl(details.url)) {
						this.tabsCache.addClientRedirect(tabId, details.url);
					}
				}
			}
		} catch (ex) {
			;
		}
	}
}