
class WarningComponent extends React.Component {
	shouldComponentUpdate(nextProps, nextState) {
		if (this.props.info.rating !== nextProps.info.rating || this.props.info.protection !== nextProps.info.protection || this.props.info.notWhitelisted !== nextProps.info.notWhitelisted || this.props.info.type !== nextProps.info.type || this.props.info.level !== nextProps.info.level || this.props.info.parental !== nextProps.info.parental) {
			return true;
		}
		return false;
	}
	goToMainPage() {
		webextApi.runtime.sendMessage({
			name: SEND_MESSAGE, params: {
				[atob("ZWM=")]: "WarningScreen",
				[atob("ZWE=")]: (this.props.info.parental ? "Blocked" : "Severe") + " WOT Logo",
				[atob("ZWw=")]: this.props.info.host
			}
		});
		webextApi.runtime.sendMessage({ name: OPEN_TAB_MESSAGE, url: MAIN_PAGE_URL + WARNING_REDIRECT_PARAMETERS });
	}
	sendOnce(action) {
		if (!window.eventSended) {
			webextApi.runtime.sendMessage({
				name: SEND_MESSAGE, params: {
					[atob("ZWM=")]: "WarningScreen",
					[atob("ZWE=")]: action,
					[atob("ZWw=")]: this.props.info.host
				}
			});
			window.eventSended = true;
		}
	}
	render() {
		var _props = this.props;
		let info = _props.info,
		    dispatch = _props.dispatch;

		let classBg = 'container-background';
		if (info.protection && info.level !== WARNINGS_LEVEL_OFF && info.type) {
			//Severe or parental warning
			if ((info.type.indexOf('severe') !== -1 || info.type.indexOf('parental') !== -1) && info.notWhitelisted && (info.safety === NOT_SAFE || info.parental && info.childSafety === NOT_SAFE || info.type === 'severe+caution')) {
				if (info.parental) {
					classBg += ' container-background_blocked';
					this.sendOnce("Parental Shown");
				} else {
					this.sendOnce("Severe Shown");
				}
				return React.createElement(
					"div",
					{ className: classBg },
					React.createElement(
						"div",
						{ className: "main-warning" },
						React.createElement(
							"div",
							{ className: "main-warning__header" },
							React.createElement("div", { onClick: this.goToMainPage.bind(this), className: "wot-logo" })
						),
						React.createElement(WarningMessageContainer, { type: info.type, parent: info.parental }),
						React.createElement(WarningReportsContainer, { type: info.type, categories: info.categories, parent: info.parental,
							host: info.host }),
						React.createElement(WarningActionsContainer, { dispatch: dispatch, host: info.host, parent: info.parental })
					)
				);
			}
			//Custom rating warning
			if (info.type.indexOf('caution') != -1 && info.notWhitelisted && info.customRating > -1 && info.customRating <= 60 && !window.DONTSHOWWARNING) {
				classBg += ' bad-rating-notify';
				return React.createElement(
					"div",
					{ className: classBg },
					React.createElement(BadRatingNotification, { dispatch: dispatch, host: info.host, rating: info.customRating })
				);
			}
			//Strip warning
			if (info.type.indexOf('strip') != -1 && info.notWhitelisted && info.rating > 0 && info.rating <= MAX_YELLOW_STRIP_RATING && info.level !== WARNINGS_LEVEL_NORMAL && info.confidence >= MIN_REPUTATION_CONFIDENCE) {
				let color = info.rating > MAX_RED_STRIP_RATING ? "Yellow" : "Red";
				classBg += ' container-background_' + color.toLowerCase();
				this.sendOnce(color + " Strip Shown");
				return React.createElement(
					"div",
					{ className: classBg },
					React.createElement(WarningStrip, { color: color, dispatch: dispatch, rating: info.rating, host: info.host })
				);
			}
			return React.createElement("i", null);
		} else {
			return React.createElement("i", null);
		}
	}
}
WarningComponent.propTypes = {};
function mapStateToPros(state) {
	let host = WotApi.getEncodedDomain(window.location.href);
	if (!window.WARNINGTYPE && window.WARNINGTYPE !== '') {
		window.WARNINGTYPE = getDonutColor(state.ratings[host], state.reviews[state.user && state.user.uid || 1][host]).warning;
	}
	let type = window.WARNINGTYPE;
	if (state.ratings[host]) {
		if (state.reviews[state.user && state.user.uid || 1][host].step < LAST_STEP) {
			window.DONTSHOWWARNING = true;
		}
		return {
			info: {
				protection: state.settings.protection,
				rating: state.ratings[host].rating,
				customRating: state.reviews[1][host] && state.reviews[1][host].step >= LAST_STEP ? state.reviews[1][host].rating : -1,
				confidence: state.ratings[host].confidence,
				parental: state.settings.parental,
				level: state.settings.warnings,
				safety: state.ratings[host].safety,
				childSafety: state.ratings[host].childSafety,
				categories: state.ratings[host].categories,
				host: host,
				type: type,
				notWhitelisted: state.whitelist.indexOf(host) === -1 && state.templist.indexOf(host) === -1
				// update: !(state.lastAction.type === SAVE_TABS)
			}
		};
	} else {
		return {
			info: {
				protection: state.settings.protection,
				rating: '',
				customRating: state.reviews[1][host] && state.reviews[1][host].step >= LAST_STEP ? state.reviews[1][host].rating : -1,
				confidence: 0,
				parental: state.settings.parental,
				level: state.settings.warnings,
				safety: UNKNOWN,
				categories: '',
				host: host,
				type: type,
				notWhitelisted: false
				// update: !(state.lastAction.type === SAVE_TABS)
			}
		};
	}
}
WarningComponent = ReactRedux.connect(mapStateToPros)(WarningComponent);