
class SerpHandler {
	constructor(store) {
		this.store = store;
	}
	init() {
		let serps = this.store.getState().serps,
		    host = WotApi.getEncodedDomain(window.location.href),
		    canBeSerp = false;
		for (let serp of serps) {
			if (new RegExp(serp.domain).test(host)) {
				canBeSerp = true;
				// if (!this.unsubscribe) {
				// 	this.unsubscribe = this.store.subscribe(this.init.bind(this));
				// }
				if (new RegExp(serp.pattern).test(window.location.href)) {
					if (!document.querySelector('.' + POPUP_CONTAINER)) {
						let container = document.createElement("div");
						container.className = POPUP_CONTAINER;
						document.body.insertBefore(container, document.body.firstChild);
					}
					clearInterval(this.interval);
					this.oldLocation = location.href;
					this.interval = setInterval((() => {
						if (location.href !== this.oldLocation) {
							this.oldLocation = location.href;
							document.querySelector('.' + POPUP_CONTAINER).innerHTML = "";
						}
					}).bind(this), 300);
					this.styles = serp.styles;
					this.ignoreUrl = serp.ignoreUrl;
					this.ignoreParent = serp.ignoreParent;
					this.ignoreChild = serp.ignoreChild;
					this.rules = serp.rules;
					this.serpid = serp.id;
					this.alreadyRendered = false;
				}
			}
		}
		if (!!this.rules) {
			this.interval = setInterval(() => {
				let activeTab = _.filter(this.store.getState().tabs, tab => {
					return tab.active;
				})[0];
				if (activeTab.url === window.location.href) {
					this.render();
				}
			}, SERP_TRACK_PERIOD);
			this.render();
		}
	}
	render() {
		for (let rule of this.rules) {
			let areas = Array.from(document.querySelectorAll(rule.areaSelector)),
			    urls = [];
			for (let area of areas) {
				if (!area.getAttribute(DONUT_ATTRIBUTE)) {
					let ignore = false;
					if (this.ignoreParent) {
						for (let parentSelector of this.ignoreParent) {
							let _area = area.parentNode,
							    restricted = document.querySelectorAll(parentSelector);
							while (_area) {
								if (Array.from(restricted).indexOf(_area) !== -1) {
									ignore = true;
								}
								_area = _area.parentNode;
							}
						}
					}
					let link = rule.linkSelector ? area.querySelector(rule.linkSelector) : area;
					if (this.ignoreChild) {
						for (let childSelector of this.ignoreChild) {
							if (link.querySelectorAll(childSelector).length > 0) {
								ignore = true;
							}
						}
					}
					/*if(area.clientHeight <= 5 || area.clientWidth <= 5) {
     	ignore = true;
     }*/
					if (link && !ignore) {
						let url = link.href || '',
						    places;
						if (rule.placeSelector) {
							places = Array.from(area.querySelectorAll(rule.placeSelector));
						} else {
							places = [area];
						}
						if (rule.getParam) {
							let match = url.match('[?&]' + rule.getParam + '=([^&]+)');
							url = decodeURIComponent(match && match.length >= 2 ? match[1] : url);
						}
						if (rule.fromString) {
							let match = new RegExp(rule.fromString).exec(link.textContent);
							url = match ? match[0] : url;
						}
						if (rule.beforeUrl) {
							url = rule.beforeUrl + url;
						}
						for (let place of places) {
							if (url && (SUPPORTED_PROTOCOLS.test(url) || url.indexOf('www.') === 0) && url.indexOf(window.location.href) !== 0) {
								if (this.ignoreUrl && new RegExp(this.ignoreUrl).test(url)) {
									continue;
								}
								let container = document.createElement("div");
								container.id = makeid();
								container.className = DONUT_CONTAINER;
								container.setAttribute("style", rule.styles || this.styles);
								if (place.textContent.trim().length == 0 && !rule.images) {
									place.style.position = "relative";
									place.style.display = "inline-block";
									container.style.position = "absolute";
									container.style.top = "0";
									container.style.right = "0";
									container.style.margin = "0";
									place.appendChild(container);
								} else {
									switch (rule.action) {
										case "appendChild":
											place.appendChild(container);
											break;
										case "insertAfter":
											place.parentNode.insertBefore(container, place.nextSibling);
											break;
										default:
											break;
									}
								}
								urls.push(url);
								let isOnboarding = this.store.getState().onboarding.step < 3 && [0, 1, 3, 9, 15].indexOf(+this.serpid) !== -1;
								ReactDOM.render(React.createElement(
									ReactRedux.Provider,
									{ store: this.store },
									React.createElement(DonutComponent, { store: this.store, target: url, serpid: this.serpid, containerSelector: '#' + container.id, isOnboarding: isOnboarding, inactive: isOnboarding && this.alreadyRendered })
								), document.querySelector('#' + container.id));
								this.alreadyRendered = true;
								rule.urlCanChange && area.setAttribute(DONUT_ATTRIBUTE, "true");
							}
						}
						!rule.urlCanChange && area.setAttribute(DONUT_ATTRIBUTE, "true");
					}
				}
			}
			urls.length > 0 && this.store.dispatch(getMultipleRatings(urls));
		}
	}
}